-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local Menu = ClarityLib:NewLib("Clarity4_1.Frames.Menu", 1)
if not Menu then return end
-- lua api
-- blizzard api
-- libs
local MenuList = ClarityLib:GetLib("Clarity4_1.Frames.MenuList")
-- locals
local Public = {}
local PublicButton = {}
local Scripts = {}
local Private = {}


function Menu.Attach(menu)
    for k,v in pairs(Public) do
        menu[k] = v
    end

    for k,v in pairs(PublicButton) do
        menu.Button[k] = v
    end

    for k,v in pairs(Scripts) do
        menu.Button:SetScript(k, v)
    end

    local width = menu:GetWidth() - menu.Button:GetWidth() - 14
    menu.Text:SetWidth(width)
    menu.Text:SetHeight(10)

    return menu
end


function Public.MakeMultiSelect(self)
    self.IsMultiSelect = true
    self.MultiSelection = {}
    return self
end


function Public.SelectIndex(self, index)
    self.MenuInfo[index].Checked = true
    if self.IsMultiSelect then
        self.MultiSelection[index] = self.MenuInfo[index].Value or index
        self:SetText()
    else
        if self.SelectedIndex and self.MenuInfo[self.SelectedIndex] then
            self.MenuInfo[self.SelectedIndex].Checked = false
        end
        self.SelectedIndex = index
        self.MenuInfo[index].Checked = true
        self:SetText(self.MenuInfo[index].Text or "")
    end
end


function Public.SelectValue(self, value)
    for index,info in ipairs(self.MenuInfo) do
        if info.Value and info.Value == value then
            self:SelectIndex(index)
        end
    end
end


function Public.SetText(self, text)
    if self.TitleText then
        text = self.TitleText
    elseif self.IsMultiSelect then
        local count = 0
        for k,v in pairs(self.MultiSelection) do
            count = count + 1
        end
        -- todo: localize
        if count > 0 then
            text = count.." Selected"
        else
            text = "None"
        end
    else
        text = text or ""
    end
    self.Text:SetText(text)
    return self
end


function PublicButton.Unlock(self)
    self.DropDown = nil
    self:UnlockHighlight()
end


function Scripts.OnClick(self, mouseButton, isDown)
    if self.DropDown then
        self.DropDown:Hide()
        self:Unlock()
    else
        local menu = self:GetParent()
        local dropDown = MenuList.GetMenu()
        dropDown:ClearAllPoints()
        dropDown:SetPoint("TopRight", self, "BottomRight", 8, 0)
        dropDown.IsMultiSelect = menu.IsMultiSelect
        dropDown.TimeToHide = menu.TimeToHide
        dropDown:Initialize(menu.MenuInfo)
        dropDown.MenuButton = self
        dropDown.Menu = menu
        dropDown:Show()
        self.DropDown = dropDown
        self:LockHighlight()
    end
    PlaySound("igMainMenuOptionCheckBoxOn")
end


function Scripts.OnHide(self)
    if self.DropDown then
        self.DropDown:Hide()
        self:Unlock()
    end
end
