-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local TabButton = ClarityLib:NewLib("Clarity4_1.Frames.TabFrame.Tab", 1)
if not TabButton then return end

-- lua api
local _G = _G
local pairs = pairs
-- blizzard api
local CreateFrame = CreateFrame
-- libs
-- locals
local Public = {}
local Scripts = {}
local Private = {}
local BlizzardDisable = nil
local BlizzardEnable = nil


-- creates a tab used by the TabFrame
function TabButton.Create(name, parent)
    local button = CreateFrame("Button", name, parent, "Clarity4_1TabButtonTemplate")

    BlizzardDisable = BlizzardDisable or button.Disable
    BlizzardEnable = BlizzardEnable or button.Enable

    button:SetNormalFontObject(GameFontNormalSmall)
    button:SetHighlightFontObject(GameFontHighlightSmall)
    button:SetDisabledFontObject(GameFontHighlightSmall)

    button.Textures =
    {
        LeftDisabled = _G[name.."LeftDisabled"],
        MiddleDisabled = _G[name.."MiddleDisabled"],
        RightDisabled = _G[name.."RightDisabled"],
        Left = _G[name.."Left"],
        Middle = _G[name.."Middle"],
        Right = _G[name.."Right"],
        HighlightTexture = _G[name.."HighlightTexture"]
    }
    button.SideWidths = button.Textures.Left:GetWidth() * 2
    button.Text = _G[name.."Text"]

    for k,v in pairs(Scripts) do
        button:SetScript(k, v)
    end

    for k,v in pairs(Public) do
        button[k] = v
    end

    button:Show()
    return button
end


-- adjusts the size of the button based on the parameters passed in
function Public.Resize(self, padding, absoluteSize, minWidth, maxWidth, absoluteTextSize)
    local tabWidth
    local textWidth
    local width
    if absoluteTextSize then
        textWidth = absoluteTextSize
    else
        self.Text:SetWidth(0)
        textWidth = self.Text:GetWidth()
    end
    if absoluteSize then
        if absoluteSize < self.SideWidths then
            width = 1
            tabWidth = self.SideWidths
        else
            width = absoluteSize - self.SideWidths
            tabWidth = absoluteSize
        end
        self.Text:SetWidth(width)
    else
        padding = padding or 24
        width = textWidth + padding
        if maxWidth and width > maxWidth then
            width = maxWidth + padding
            self.Text:SetWidth(width)
        else
            self.Text:SetWidth(0)
        end
        if minWidth and width < minWidth then
            width = minWidth
        end
        tabWidth = width + self.SideWidths
    end
    self.Textures.Middle:SetWidth(width)
    self.Textures.MiddleDisabled:SetWidth(width)

    self:SetWidth(tabWidth)
    self.Textures.HighlightTexture:SetWidth(tabWidth)
end


-- returns the adjusted width of the tab button
function Public.GetWidth(self)
    return self:GetTextWidth() + self.SideWidths
end


-- disables the tab, making it un-clickable
function Public.Disable(self)
    self.IsDisabled = true
    self.Textures.Left:Show()
    self.Textures.Middle:Show()
    self.Textures.Right:Show()

    BlizzardDisable(self)
    self.SetDisabledFontObject(GameFontDisableSmall)

    self.Textures.LeftDisabled:Hide()
    self.Textures.MiddleDisabled:Hide()
    self.Textures.RightDisabled:Hide()

    self:GetParent():UpdateTabs()
end


-- enables the tab, making it clickable
function Public.Enable(self)
    self.IsDisabled = false
    self.SetDisabledFontObject(GameFontHighlightSmall)
    self:GetParent():UpdateTabs()
end


-- updates the appearance of the button when it is selected
function Public.Select(self)
    self.Textures.Left:Hide()
    self.Textures.Middle:Hide()
    self.Textures.Right:Hide()
    BlizzardDisable(self)

    -- self.Test:SetPoint("Center", self, "Center", 0, 3)

    self.Textures.LeftDisabled:Show()
    self.Textures.MiddleDisabled:Show()
    self.Textures.RightDisabled:Show()
    return self
end


-- updates the appearance of the button when it is deselected
function Public.Deselect(self)
    self.Textures.Left:Show()
    self.Textures.Middle:Show()
    self.Textures.Right:Show()
    BlizzardEnable(self)

    -- self.Test:SetPoint("Center", self, "Center", 0, 2)

    self.Textures.LeftDisabled:Hide()
    self.Textures.MiddleDisabled:Hide()
    self.Textures.RightDisabled:Hide()
    return self
end


function Scripts.OnClick(self, mouseButton, isDown)
    self:GetParent().SelectedTab = self.ID
    self:GetParent():UpdateTabs()
end
