-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local TabFrame = ClarityLib:NewLib("Clarity4_1.Frames.TabFrame", 1)
if not TabFrame then return end

-- lua api
local ipairs = ipairs
local pairs = pairs
-- blizzard api
local CreateFrame = CreateFrame
-- libs
local TabButton = ClarityLib:GetLib("Clarity4_1.Frames.TabFrame.Tab")
-- locals
local Public = {}
local Private = {}


-- creates a frame that has tabs attached to it.  Selecting a tab will hide a the contents
-- of other tabs, and show the contents of the selected tab
function TabFrame.Create(name, parent)
    local frame = CreateFrame("Frame", name, parent)
    return TabFrame.Attach(frame)
end


function TabFrame.Attach(frame)
    frame.Tabs = {}
    frame.Frames = {}

    for k,v in pairs(Public) do
        frame[k] = v
    end
    return frame
end


-- adds a tab to the main frame.
-- tabText: the txt that will appear in the tab
-- frameOrInherit: a frame, or virtual frame template name
-- a frame is automatically created if nil or an invalid value is passed in
function Public.AddTab(self, tabText, frameOrInherit)
    local index = #self.Tabs+1
    local frame = Private.GetFrame(self, frameOrInherit, index)
    local tabButton = TabButton.Create(self:GetName().."Tab"..index, self)
    tabButton:SetText(tabText)
    tabButton.ID = index
    self.Tabs[index] = tabButton
    self.Frames[index] = frame
    frame:SetAllPoints(self)
    if index == 1 then
        tabButton:SetPoint("BottomLeft", self, "TopLeft", 0, 0)
    else
        tabButton:SetPoint("BottomLeft", self.Tabs[index-1], "BottomRight", 0, 0)
    end
    return frame
end


-- causes all tabs to be resized based on parameters passed in
function Public.ResizeTabs(self, padding, absoluteSize, minWidth, maxWidth, absoluteTextSize)
    for index,tab in ipairs(self.Tabs) do
        tab:Resize(padding, absoluteSize, minWidth, maxWidth, absoluteTextSize)
    end
    return self
end


-- selects a specific tab, and thus activates its specific sub-frame
function Public.SelectTab(self, id)
    self.SelectedTab = id
    self:UpdateTabs()
end


-- updates the appearance of each tab, hides unselected sub-frames, and shows the
-- selected sub-frame
function Public.UpdateTabs(self)
    if self.SelectedTab then
        for index,tab in ipairs(self.Tabs) do
            if tab.IsDisabled then
                tab:Disable()
                self.Frames[index]:Hide()
            elseif index == self.SelectedTab then
                tab:Select()
                self.Frames[index]:SetFrameLevel(self:GetFrameLevel()+1)
                self.Frames[index]:Show()
            else
                tab:Deselect()
                self.Frames[index]:Hide()
            end
        end
    end
end


function Private.GetFrame(self, frameOrInherit, index)
    if frameOrInherit then
        if frameOrInherit.GetObjectType and frameOrInherit:GetObjectType() == "Frame" then
            frameOrInherit:SetParent(self)
            return frameOrInherit
        end
        if type(frameOrInherit) ~= "string" then
            frameOrInherit = nil
            error("TabFrame requires a frame or frame type to be passed in.")
        end
    end
    return CreateFrame("Frame", self:GetName().."SubFrame"..index, self, frameOrInherit)
end
