-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local AddOnName, AddOn = ...
local ClarityLibName, Version = "ClarityLib", 2
local ClarityLib = _G[ClarityLibName]

if not ClarityLib or ClarityLib.Version < Version then
-- lua api
local setmetatable = setmetatable
-- blizzard api
local wipe = wipe

    ClarityLib = ClarityLib or { __NameSpaces={}, __AddOns={}, __Versions={} }
    _G[ClarityLibName] = ClarityLib
    ClarityLib.Version = Version
    -- global singleton, so no need for meta table for attaching functions

    -- determines if a lib with the version number is new (returns true)
    -- already exists or older versions will return false
    local function CheckVersionIsNew(self, name, newVersion)
        if not self.__Versions[name] or newVersion > self.__Versions[name] then
            self.__Versions[name] = newVersion
            return true
        end
        return false
    end


    -- returns a table to put the lib info into if it is a new lib
    -- returns nil if the current lib, or a later version already exists
    function ClarityLib.NewLib(self, name, version)
        if CheckVersionIsNew(self, name, version) then
            self.__NameSpaces[name] = self.__NameSpaces[name] or {}
            -- cleans the old table and returns it
            -- this way any existing references are updated with
            -- the new version
            return wipe(setmetatable(self.__NameSpaces[name], nil))
        end
        return nil
    end


    local function CreateErrorMessage(...)
        local names = {...}
        local errorMessage = "Global Library Missing: ".. names[1]
        for index = 2,#names do
            errorMessage = errorMessage.."."..names[index]
        end
        return errorMessage
    end


    -- will call error if the library does not exist
    function ClarityLib.GetLib(self, name)
        local nameSpace = self.__NameSpaces[name]
        if nameSpace == nil then
            local errorMessage = CreateErrorMessage(name)
            error(errorMessage)
        end
        return nameSpace
    end
    ClarityLib.GetLibStrict = ClarityLib.GetLib -- for backwards compatibility
end

ClarityLib.__AddOns[AddOnName] = AddOn
