-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local NameSpace = ClarityLib:NewLib("Clarity4_1.Meta.NameSpace", 1)
local AddOnName, AddOn = ...
if not NameSpace then
    NameSpace = ClarityLib:GetLib("Clarity4_1.Meta.NameSpace")
    AddOn.NameSpace = NameSpace.CreateNameSpaceMaster()
    return
end
-- lua api
local error = error
local ipairs = ipairs
local setmetatable = setmetatable
-- blizzard api
-- libs
-- locals


local NameSpaceMeta = {}
NameSpaceMeta.__index = NameSpaceMeta


local function GetAddOnNameSpace(self, ...)
    local nameSpace = self.__NameSpaces
    local prevNameSpace = nil
    for _,name in ipairs({...}) do
        if nameSpace.IsLazy then
            -- todo: parse ... to generate namespace
            if prevNameSpace then
                name = prevNameSpace.."."..name
            end
            error("Accessing a sub-name space in an lazy object will evaluate the lazy object.  NameSpace: "..name)
        end
        if nameSpace[name] then
            nameSpace = nameSpace[name]
        else
            return nil
        end
        prevNameSpace = name
    end
    return nameSpace
end


local function CreateNameSpace(self, ...)
    local nameSpace = self.__NameSpaces
    local prevNameSpace = nil
    for _,name in ipairs({...}) do
        if nameSpace.IsLazy then
            if prevNameSpace then
                name = prevNameSpace.."."..name
            end
            error("Creating a sub-namespace in an lazy object will evaluate the lazy object.  NameSpace: "..name)
        end
        nameSpace[name] = nameSpace[name] or {}
        nameSpace = nameSpace[name]
        prevNameSpace = name
    end
    return nameSpace
end


-- tries to get a namespace from the addon
-- else creates a new namespace
function NameSpaceMeta.__call(self, ...)
    -- try to get AddOn version
    local nameSpace = GetAddOnNameSpace(self, ...)
    -- create AddOn version
    return nameSpace or CreateNameSpace(self, ...)
end


local function CreateErrorMessage(...)
    local total = select('#', ...)
    local errorMessage = "NameSpace Missing: ".. select(1, ...)
    for index = 2,total do
        errorMessage = errorMessage.."."..select(index, ...)
    end
    return errorMessage
end


-- tries to get a namespace.  If the namespace does not already exist then
-- an error will be generated
function NameSpaceMeta.Strict(self, ...)
    local nameSpace = GetAddOnNameSpace(self, ...)
    if nameSpace == nil then
        local errorMessage = CreateErrorMessage(...)
        error(errorMessage)
    end
    return nameSpace
end


-- creates the NameSpace object
-- this is automatically attached to the add on object
function NameSpace.CreateNameSpaceMaster()
    return setmetatable({__NameSpaces={}}, NameSpaceMeta)
end


AddOn.NameSpace = NameSpace.CreateNameSpaceMaster()
