-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local Utility = ClarityLib:NewLib("Clarity4_1.Meta.Utility", 1)
if not Utility then return end
-- lua api
local Max = math.max
local Min = math.min
-- blizzard api
-- libs
-- locals


-- returns value where min <= value <= max
function Utility.Constrain(min, value, max)
    return Max(min, Min(value, max))
end


-- copies all values from defaults onto overrides if a corresponding key on overrides
-- does not already exist
function Utility.CopyOverrides(defaults, overrides)
    overrides = overrides or {}
    for k,v in pairs(defaults) do
        overrides[k] = overrides[k] or v -- use override or fallback to default
    end
    return overrides
end


-- shallow copies a table.  Sub-tables are not copied at all
function Utility.ShallowCopy(from)
    local to = {}
    for k,v in pairs(from) do
        if type(v) ~= "table" then
            to[k] = v
        end
    end
    return to
end


-- deep copies a table
-- values in "to" that are not in "from" are not overwritten
-- does not copy or set the metatable
-- has a safety catch to prevent recursive tables, but this can
-- block other patterns e.g. tables in diamond formation
function Utility.DeepCopy(from, to, skip)
    to = to or {}
    skip = skip or {}
    skip[from] = true
    for k,v in pairs(from) do
        if type(v) == "table" then
            if not skip[v] then
                to[k] = Utility.DeepCopy(v, to[k], skip)
            end
        else
            to[k] = v
        end
    end
    return to
end
