-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Floor = math.floor
local ipairs = ipairs
local pairs = pairs
local tostring = tostring
local type = type
-- blizzard api
local CreateFrame = CreateFrame
-- libs
-- add on api
local L = DataDump.Localization()
local Simple = DataDump.NameSpace("FrameDisplay", "Editor", "Simple")
local VarData = DataDump.NameSpace("VarDisplay", "Data")
local VarDisplay = DataDump.NameSpace("VarDisplay", "Display")
-- local name space
local ScriptsTab = DataDump.NameSpace("FrameDisplay", "Scripts")
local Public = {}
local LineGetter = {}
local LineSetters = {}
local ScriptsGetScripts = {}
local Private = {}
-- locals
local _kSpacing = 2
local _kMaxLines = 10
local _kScripts =
{
    OnAnimFinished = "OnAnimFinished(self)",
    OnArrowPressed = "OnArrowPressed(self, key)",
    OnAttributeChanged = "OnAttributeChanged(self, name, value)",
    OnButtonUpdate = "OnButtonUpdate(self, action)",
    OnChar = "OnChar(self, text)",
    OnCharComposition = "OnCharComposition(?)",
    OnClick = "OnClick(self, mouseButton, isDown)",
    OnColorSelect = "OnColorSelect(self, r, g, b)",
    OnCursorChanged = "OnCursorChanged(self, x, y, w, h)",
    OnDisable = "OnDisable(self)",
    OnDoubleClick = "OnDoubleClick(self, mouseButton)",
    OnDragStart = "OnDragStart(self, button)",
    OnDragStop = "OnDragStop(self)",
    OnEditFocusGained = "OnEditFocusGained(self)",
    OnEditFocusLost = "OnEditFocusLost(self)",
    OnEnable = "OnEnable(self)",
    OnEnter = "OnEnter(self)",
    OnEnterPressed = "OnEnterPressed(self)",
    OnError = "OnError(?)",
    OnEscapePressed = "OnEscapePressed(self)",
    OnEvent = "OnEvent(self, event, ...)",
    OnExternalLink = "OnExternalLink(?)",
    OnHide = "OnHide(self)",
    OnHorizontalScroll = "OnHorizontalScroll(?)",
    OnHyperlinkClick = "OnHyperlinkClick(?)",
    OnHyperlinkEnter = "OnHyperlinkEnter(?)",
    OnHyperlinkLeave = "OnHyperlinkLeave(?)",
    OnInputLanguageChanged = "OnInputLanguageChanged(?)",
    OnJoystickAxisMotion = "OnJoystickAxisMotion(?)",
    OnJoystickButtonDown = "OnJoystickButtonDown(?)",
    OnJoystickButtonUp = "OnJoystickButtonUp(?)",
    OnJoystickHatMotion = "OnJoystickHatMotion(?)",
    OnJoystickStickMotion = "OnJoystickStickMotion(?)",
    OnKeyDown = "OnKeyDown(self, key)",
    OnKeyUp = "OnKeyUp(self, key)",
    OnLeave = "OnLeave(self, motion)",
    OnLoad = "OnLoad(self)",
    OnMessageScrollChanged = "OnMessageScrollChanged(?)",
    OnMinMaxChanged = "OnMinMaxChanged(?)",
    OnMouseDown = "OnMouseDown(self, mouseButton)",
    OnMouseUp = "OnMouseUp(self, mouseButton)",
    OnMouseWheel = "OnMouseWheel(self, delta)",
    OnMovieFinished = "OnMovieFinished(?)",
    OnMovieHideSubtitle = "OnMovieHideSubtitle(?)",
    OnMovieShowSubtitle = "OnMovieShowSubtitle(?)",
    OnReceiveDrag = "OnReceiveDrag(self)",
    OnScrollRangeChanged = "OnScrollRangeChanged(?)",
    OnShow = "OnShow(self)",
    OnSizeChanged = "OnSizeChanged(self, width, height)",
    OnSpacePressed = "OnSpacePressed(self)",
    OnTabPressed = "OnTabPressed(self)",
    OnTextChanged = "OnTextChanged(self)",
    OnTextSet = "OnTextSet(self)",
    OnTooltipAddMoney = "OnTooltipAddMoney(self, cost, maxCost)",
    OnTooltipCleared = "OnTooltipCleared(self)",
    OnTooltipSetAchievement = "OnTooltipSetAchievement(?)",
    OnTooltipSetDefaultAnchor = "OnTooltipSetDefaultAnchor(self, parent)",
    OnTooltipSetItem = "OnTooltipSetItem(self)",
    OnTooltipSetQuest = "OnTooltipSetQuest(?)",
    OnTooltipSetSpell = "OnTooltipSetSpell(?)",
    OnTooltipSetUnit = "OnTooltipSetUnit(self)",
    OnUpdate = "OnUpdate(self, elapsed)",
    OnUpdateModel = "OnUpdateModel(self)",
    OnValueChanged = "OnValueChanged(self, value)",
    OnVerticalScroll = "OnVerticalScroll(self, offset, lineHeight, updateFunction)",
    PostClick = "PostClick(self, mouseButton, isDown)",
    PreClick = "PreClick(self, mouseButton, isDown)",
}
local _kScriptsSorted =
{
    "OnAnimFinished",
    "OnArrowPressed",
    "OnAttributeChanged",
    "OnButtonUpdate",
    "OnChar",
    "OnCharComposition",
    "OnClick",
    "OnColorSelect",
    "OnCursorChanged",
    "OnDisable",
    "OnDoubleClick",
    "OnDragStart",
    "OnDragStop",
    "OnEditFocusGained",
    "OnEditFocusLost",
    "OnEnable",
    "OnEnter",
    "OnEnterPressed",
    "OnError",
    "OnEscapePressed",
    "OnEvent",
    "OnExternalLink",
    "OnHide",
    "OnHorizontalScroll",
    "OnHyperlinkClick",
    "OnHyperlinkEnter",
    "OnHyperlinkLeave",
    "OnInputLanguageChanged",
    "OnJoystickAxisMotion",
    "OnJoystickButtonDown",
    "OnJoystickButtonUp",
    "OnJoystickHatMotion",
    "OnJoystickStickMotion",
    "OnKeyDown",
    "OnKeyUp",
    "OnLeave",
    "OnLoad",
    "OnMessageScrollChanged",
    "OnMinMaxChanged",
    "OnMouseDown",
    "OnMouseUp",
    "OnMouseWheel",
    "OnMovieFinished",
    "OnMovieHideSubtitle",
    "OnMovieShowSubtitle",
    "OnReceiveDrag",
    "OnScrollRangeChanged",
    "OnShow",
    "OnSizeChanged",
    "OnSpacePressed",
    "OnTabPressed",
    "OnTextChanged",
    "OnTextSet",
    "OnTooltipAddMoney",
    "OnTooltipCleared",
    "OnTooltipSetAchievement",
    "OnTooltipSetDefaultAnchor",
    "OnTooltipSetItem",
    "OnTooltipSetQuest",
    "OnTooltipSetSpell",
    "OnTooltipSetUnit",
    "OnUpdate",
    "OnUpdateModel",
    "OnValueChanged",
    "OnVerticalScroll",
    "PostClick",
    "PreClick",
}


function ScriptsTab.Create(parent)
    local name = parent:GetName().."ScriptsTab"
    local displayTab = CreateFrame("Frame", name, parent, "DataDumpScriptsTabTemplate")
    displayTab:SetAllPoints()
    displayTab:SetFrameLevel(parent:GetFrameLevel()+1)

    for funcName,line in pairs(displayTab.Settings) do
        if L[funcName] then
            line.Title:SetText(L[funcName])
        end
        if LineSetters[funcName] then
            LineSetters[funcName](line, funcName)
        end
    end

    displayTab.GetScripts:SetText(L["GetScripts"])
    for k,v in pairs(ScriptsGetScripts) do
        displayTab.GetScripts:SetScript(k, v)
    end

    for k,v in pairs(Public) do
        displayTab[k] = v
    end

    return displayTab
end


function Public.LoadFrame(self, frame, forceRefresh)
    for funcName,line in pairs(self.Settings) do
        if L[funcName] then
            if frame[funcName] then
                local getter = LineGetter[funcName] or LineGetter.Default
                getter(line, funcName, frame)
                line:Show()
            else
                line:Hide()
            end
        end
    end

    if frame.HasScript then
        self.GetScripts.dataFrame = frame
        self.GetScripts:Enable()
    else
        self.GetScripts:Disable()
    end
end


function Public.ClearValue(self)
    Simple.ClearAllValues(self.Settings)
end


function LineGetter.Default(line, func, frame)
    line.Value:SetText(tostring(frame[func](frame)))
end


function ScriptsGetScripts.OnClick(self, mouse, isDown)
    if self.dataFrame then
        local varDisplay = VarDisplay.Manager:GetDisplay()
        local name = (self.dataFrame:GetName() or tostring(self.dataFrame)).." • Scripts"
        varDisplay.VarInput:SetText(name)
        varDisplay.VarInput:Disable()
        varDisplay.noSave = true

        local value = Private.GetScripts(self.dataFrame)
        local varData = VarData.CreateWithValue(value, name)
        varDisplay:SetCrumb(varData, 1)
        varDisplay:Show()
    end
end


function Private.GetScripts(frame)
    local scriptTable = {}
    for scriptName,textValue in pairs(_kScripts) do
        if frame:HasScript(scriptName) then
            scriptTable[_kScripts[scriptName]] = frame:GetScript(scriptName) and "yes" or ""
        end
    end
    return scriptTable
end
