-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local pairs = pairs
-- blizzard api
local CreateFrame = CreateFrame
-- libs
local Lazy = ClarityLib:GetLib("Clarity4_1.Meta.Lazy")
-- add on api
-- local name space
local VarBreadCrumb = DataDump.NameSpace("VarDisplay", "BreadCrumb")
local Public = {}
local ScriptsUp = {}
local Private = {}
-- locals
local _kSpacing = 2
local _kMaxLines = 10


function VarBreadCrumb.Initialize(parent)
    local line = parent.VarBreadCrumbs[1]
    Private.InitializeLine(line, 1)

    for index = 2,_kMaxLines do
        line = Private.Create(parent, index, line)
        parent.VarBreadCrumbs[index] = line
    end
end


function VarBreadCrumb.SetInput(parent, data, index)
    if index > _kMaxLines then
        return
    end
    data:Eval(true)
    parent.ActiveInput = index
    parent.VarBreadCrumbs[index]:SetData(data)
    parent.VarBreadCrumbs[index]:Show()
    Private.UpdateLines(parent.VarBreadCrumbs, index)
    if type(data.value) == "table" then
        data:Expand()
    end
end


function VarBreadCrumb.SetNextInput(parent, data)
    if not parent.ActiveInput then
        VarBreadCrumb.SetInput(parent, data, 1)
    else
        VarBreadCrumb.SetInput(parent, data, parent.ActiveInput + 1)
    end
end


function VarBreadCrumb.IsError(parent)
    local index = parent.ActiveInput
    if not parent.VarBreadCrumbs[index].data
    or parent.VarBreadCrumbs[index].data.parseError then
        return true
    end
    return false
end


function Public.SetData(self, data)
    self.data = data
    self.Text:SetText(data.key)
end


function ScriptsUp.OnClick(self, mouseButton, isDown)
    local breadCrumb = self:GetParent()
    breadCrumb:GetParent():SetCrumb(breadCrumb.data, breadCrumb.index)
end


function Private.Create(parent, index, prevInput)
    local name = parent:GetName()
    local newInput = CreateFrame("Frame", name.."VarBreadCrumbs"..index, parent, "DataDumpVarBreadCrumbTemplate")
    newInput:SetPoint("TOPLEFT", prevInput, "BOTTOMLEFT", 0, -_kSpacing)
    newInput:SetPoint("TOPRIGHT", prevInput, "BOTTOMRIGHT", 0, -_kSpacing)
    Private.InitializeLine(newInput, index)
    return newInput
end


function Private.InitializeLine(breadCrumb, index)
    breadCrumb.index = index
    breadCrumb.Up:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    for k,v in pairs(ScriptsUp) do
        breadCrumb.Up:SetScript(k, v)
    end
    for k,v in pairs(Public) do
        breadCrumb[k] = v
    end

    local width = breadCrumb:GetWidth() - breadCrumb.Up:GetWidth()
    breadCrumb.Text:SetWidth(width)
    breadCrumb:Hide()
end


function Private.UpdateLines(breadCrumbs, start)
    for index = 1,start-1 do
        breadCrumbs[index].Up:Show()
    end
    breadCrumbs[start].Up:Hide()
    for index = start+1,_kMaxLines do
        breadCrumbs[index]:Hide()
    end
end
