-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Floor = math.floor
local pairs = pairs
local type = type
local tostring = tostring
-- blizzard api
local CreateFrame = CreateFrame
-- libs
-- add on api
local FrameDisplay = DataDump.NameSpace("FrameDisplay", "Display")
local L = DataDump.Localization()
local VarDisplay = DataDump.NameSpace("VarDisplay", "Display")
-- local name space
local VarDataLine = DataDump.NameSpace("VarDisplay", "DataLine")
local Public = {}
local ScriptsExpand = {}
local ScriptsKey = {}
local ScriptsType = {}
local ScriptsValue = {}
local ScriptsCopyText = {}
local Private = {}
-- locals
local _kSpacing = 2


function VarDataLine.UpdateDisplayLines(parent)
    local currentLines = #parent.DataLine
    local line = parent.DataLine[currentLines]
    local totalHeight = parent:GetHeight()
    local lineHeight = line:GetHeight() + _kSpacing
    local displayLines = Floor(totalHeight / lineHeight)
    parent.DisplayLines = displayLines

    for index = currentLines+1,displayLines do
        line = Private.Create(parent, index, line)
        parent.DataLine[index] = line
    end
    return displayLines ~= currentLines
end


function VarDataLine.ClearLines(varLines, start)
    for index = start,#varLines do
        varLines[index]:Hide()
    end
end


function VarDataLine.Initialize(line)
    line.Expand:RegisterForClicks("LeftButtonUp")
    for k,v in pairs(ScriptsExpand) do
        line.Expand:SetScript(k, v)
    end

    line.Key:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    for k,v in pairs(ScriptsKey) do
        line.Key:SetScript(k, v)
    end
    line.Key:SetText(" ") -- force creation of fontstring before restricting it

    line.Type:RegisterForClicks("RightButtonUp")
    for k,v in pairs(ScriptsType) do
        line.Type:SetScript(k, v)
    end
    line.Type:SetText(" ") -- force creation of fontstring before restricting it
    line.Type:GetFontString():SetWidth(line.Type:GetWidth())

    for k,v in pairs(ScriptsValue) do
        line.Value:SetScript(k, v)
    end
    line.Value:SetText(" ") -- force creation of fontstring before restricting it
    line.Value:GetFontString():SetWidth(line.Value:GetWidth())

    for k,v in pairs(Public) do
        line[k] = v
    end

    line:Hide()
end


function Public.SetData(line, data)
    local value = data.value
    local valueType = type(data.value)
    if valueType == "table" then
        if data.shouldExpand then
            line.Expand:MakeMinus()
        else
            line.Expand:MakePlus()
        end
        line.Expand:Show()
        if data.value.GetObjectType then
            valueType = data.value:GetObjectType()
        end
        if data.value.GetName then
            value = data.value:GetName() or tostring(value)
        end
    else
        line.Expand:Hide()
    end

    local indent = (data.indent or 0) + 5
    line.Key:ClearAllPoints()
    line.Key:SetPoint("Left", line.Expand, "Right", indent, 0)
    line.Key:SetPoint("Right", line.Expand, "Right", 170, 0)
    line.Key:GetFontString():SetWidth(170 - indent)

    line.Data = data
    line.Key:SetText(tostring(data.key))
    line.Type:SetText(valueType)
    line.Value:SetText(tostring(value))

    line:Show()
end


function ScriptsExpand.OnClick(self, mouseButton, isDown)
    self:Toggle()
    if self.__state then
        self:GetParent().Data:Collapse()
    else
        self:GetParent().Data.shouldExpand = true
        self:GetParent().Data:Expand()
    end
    self:GetParent():GetParent():GetParent():RefreshData(true)
end


function ScriptsKey.OnClick(self, mouseButton, isDown)
    local data = self:GetParent().Data
    if mouseButton == "LeftButton" then
        self:GetParent():GetParent():GetParent():SetNextInput(data:Copy())
    else
        local varDisplay = VarDisplay.Manager:GetDisplay()
        local name = data:GetFullName()
        if name then
            varDisplay.VarInput:SetText(name)
        else
            varDisplay.VarInput:SetText("•unnamed")
            varDisplay.VarInput:Disable()
            varDisplay.noSave = true
        end
        varDisplay:SetCrumb(data:Copy(), 1)
        varDisplay:Show()
    end
end


function ScriptsType.OnClick(self, mouseButton, isDown)
    local data = self:GetParent().Data
    data:Eval()
    local value = data.value
    if value ~= nil and type(value) == "table"
    and value.IsObjectType and value:IsObjectType("Region") then
        local name = value:GetName() or data:GetRegionName() or L["•unnamed"]
        local frameDisplay = FrameDisplay.Manager:GetDisplay()
        frameDisplay:SetFrameInput(name, value)
        frameDisplay:RefreshData(true)
        frameDisplay:Show()
    end
end


function ScriptsValue.OnClick(self, mouseButton, isDown)
    if not self.CopyText then
        self.CopyText = CreateFrame("EditBox", self:GetName().."CopyText", self, "DataDumpCopyTextTemplate")
        self.CopyText:SetPoint("Left", self, "Left", 0, 0)
        self.CopyText:SetPoint("Right", self, "Right", -(self.CopyText.Close:GetWidth()), 0)
        for k,v in pairs(ScriptsCopyText) do
            self.CopyText:SetScript(k, v)
        end
    end
    self.CopyText.__value = self:GetText()
    self.CopyText:SetText(self.CopyText.__value)
    self.CopyText:Show()
    self.CopyText:SetFocus()
end


function ScriptsValue.OnSizeChanged(self, width, height)
    self:GetFontString():SetWidth(self:GetWidth())
end


function ScriptsCopyText.OnEscapePressed(self)
    self:Hide()
end


function ScriptsCopyText.OnTextChanged(self, userInput)
    if userInput then
        self:SetText(self.__value)
    end
end


function Private.Create(parent, index, prevLine)
    local name = parent:GetName()
    local newLine = CreateFrame("Frame", name.."DataLine"..index, parent, "DataDumpVarDataLineTemplate")
    newLine:SetPoint("TopLeft", prevLine, "BottomLeft", 0, -_kSpacing)
    newLine:SetPoint("TopRight", prevLine, "BottomRight", 0, -_kSpacing)
    VarDataLine.Initialize(newLine)
    return newLine
end
