-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Floor = math.floor
local pairs = pairs
-- blizzard api
local CreateFrame = CreateFrame
-- libs
local LazyLib = ClarityLib:GetLib("Clarity4_1.Meta.Lazy")
-- add on api
local DisplayManager = DataDump.NameSpace("Common", "Manager")
local L = DataDump.Localization()
local Refresh = DataDump.NameSpace("Common", "Refresh")
local VarBreadCrumb = DataDump.NameSpace("VarDisplay", "BreadCrumb")
local VarData = DataDump.NameSpace("VarDisplay", "Data")
local VarDataLine = DataDump.NameSpace("VarDisplay", "DataLine")
local VarDisplayScroll = DataDump.NameSpace("VarDisplay", "Scroll")
-- local name space
local VarDisplay = DataDump.NameSpace("VarDisplay", "Display")
local Public = {}
local Scripts = {}
local ScriptsDelete = {}
local ScriptsVarInput = {}
local ScriptsResize = {}
local Private = {}
-- locals


function Private.Create(index)
    local newDisplay = CreateFrame("Frame", "DataDumpVarDisplay"..index, UIParent, "DataDumpVarDisplayTemplate")

    newDisplay.index = index
    -- main frame
    newDisplay.Title:SetText(L["Variable Display "]..index)

    for k,v in pairs(Public) do
        newDisplay[k] = v
    end

    newDisplay:RegisterForDrag("LeftButton")
    for k,v in pairs(Scripts) do
        newDisplay:SetScript(k, v)
    end

    Refresh.Initialize(newDisplay.RefreshButton)

    -- delete button
    newDisplay.Delete:SetText(L["Delete"])
    for k,v in pairs(ScriptsDelete) do
        newDisplay.Delete:SetScript(k, v)
    end

    -- resize
    for k,v in pairs(ScriptsResize) do
        newDisplay.Resize:HookScript(k, v)
    end

    -- var input
    for k,v in pairs(ScriptsVarInput) do
        newDisplay.VarInput:HookScript(k, v)
    end

    -- bread crumbs
    VarBreadCrumb.Initialize(newDisplay)

    -- scroll area
    VarDisplayScroll.Initialize(newDisplay.ScrollArea)

    return newDisplay
end


function Public.SetStateInfo(self, state)
    self.VarInput:SetText(state.input)

    local screenX, screenY = UIParent:GetSize()
    local x = Floor(state.x * screenX / 100)
    local y = Floor(state.y * screenY / 100)
    self:ClearAllPoints()
    self:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y)

    self:SetSize(state.w, state.h)

    self.RefreshButton:SetRefreshRate(state.frames, state.seconds)
end


function Public.GetStateInfo(self)
    self.state = self.state or {}
    self.state.isShown = self:IsShown()
    self.state.noSave = self.noSave
    self.state.input = self.VarInput:GetText()

    local screenX, screenY = UIParent:GetSize()
    local x, y = self:GetCenter()
    self.state.x = Floor(x * 100 / screenX)
    self.state.y = Floor(y * 100 / screenY)

    local w, h = self:GetSize()
    self.state.w = Floor(w)
    self.state.h = Floor(h)

    local frames, seconds = self.RefreshButton:GetRefreshRate()
    self.state.frames = frames ~= 30 and frames or nil
    self.state.seconds = seconds ~= 1 and seconds or nil

    return self.state
end


function Public.Initialize(self)
    self.VarInput:Enable()
    self.noSave = nil
    self:Clear()

    self:SetSize(560, 350)
    self:ClearAllPoints()
    self:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
end


function Public.SetCrumb(self, data, index)
    VarBreadCrumb.SetInput(self, data, index)
    self.ScrollArea:SetData(data)
    self.ScrollArea:Refresh()
end


function Public.SetNextInput(self, data)
    VarBreadCrumb.SetNextInput(self, data)
    self.ScrollArea:SetData(data)
    self.ScrollArea:Refresh()
end


-- nothing currently to force a refresh upon
function Public.RefreshData(self, forceRefresh)
    if not self.ScrollArea:Refresh(true) then
        ScriptsVarInput.OnEnterPressed(self.VarInput)
        self.VarInput:ClearFocus()
        if VarBreadCrumb.IsError(self) then
            self.RefreshButton:StopRefresh()
        end
    end
end


function Public.Clear(self)
    self.VarInput:SetText("")
    ScriptsVarInput.OnEnterPressed(self.VarInput)
    self.ScrollArea:Clear()
    self:ClearAllPoints()
    self:SetPoint("Center", UIParent, "Center", 0, 0)
end


function Scripts.OnDragStart(self, mouseButton)
    self:StartMoving()
end


function Scripts.OnDragStop(self)
    self:StopMovingOrSizing()
    VarDisplay.Manager:Save(self)
end


function Scripts.OnHide(self)
    VarDisplay.Manager:Save(self)
end


function Scripts.OnShow(self)
    local level = self:GetFrameLevel()
    if level < 3 then
        level = level + 2
        self:SetFrameLevel(level)
    end
    self.LeftBar:SetFrameLevel(level-2)
    VarDisplay.Manager:Save(self)
end


function ScriptsVarInput.OnEnterPressed(self)
    local value = self:GetText()
    local display = self:GetParent()
    VarDisplay.Manager:Save(display)
    local data = VarData.Create(value)
    Public.SetCrumb(display, data, 1)
end


function ScriptsDelete.OnClick(self, mouseButton, isDown)
    local display = self:GetParent()
    display.VarInput:SetText("")
    display:Hide()
end


function Private.Initialize()
    DataDumpData = DataDumpData or {}
    VarDisplay.Manager = DisplayManager.Init(DataDumpData, "VarDisplay", Private.Create)
end

LazyLib.MakeLazy(VarDisplay, Private.Initialize)
