-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Floor = math.floor
local TableSort = table.sort
local tostring = tostring
-- blizzard api
local CreateFrame = CreateFrame
local Wipe = wipe
-- libs
-- add on api
local VarData = DataDump.NameSpace("VarDisplay", "Data")
local VarDisplay = DataDump.NameSpace("VarDisplay", "VarDisplay")
local VarDataLine = DataDump.NameSpace("VarDisplay", "DataLine")
-- local name space
local VarDisplayScroll = DataDump.NameSpace("VarDisplay", "Scroll")
local Public = {}
local Scripts = {}
local Private = {}
-- locals
local _kSpacing = 2
local _kIndent = 15


function VarDisplayScroll.Initialize(scrollFrame)
    local name = scrollFrame:GetName()
    --scrollFrame.VarResult = { Key = "", HidePlusMinus=true }
    scrollFrame.VarData = {}
    scrollFrame.VarTable = {}
    scrollFrame.VarExpanded = {}

    for k,v in pairs(Scripts) do
        scrollFrame:SetScript(k, v)
    end

    for k,v in pairs(Public) do
        scrollFrame[k] = v
    end

    VarDataLine.Initialize(scrollFrame.DataLine[1])
    scrollFrame.DisplayLines = 1

    VarDataLine.UpdateDisplayLines(scrollFrame)
    scrollFrame.Scroll:SetCallback(Private.ScrollOnValueChanged, scrollFrame)
    scrollFrame.Scroll:UpdateScrollRange(scrollFrame.DisplayLines, 0)
end


function Public.SetData(self, data)
    self.data = data
    Public.Refresh(self, true)
end


function Public.Refresh(self, forceRefresh)
    if self.data then
        self.DisplayInfo = self.data:FlattenOutput(forceRefresh)
        self:Update()
        return true
    end
    return false
end


function Public.Update(self, offset, indent)
    local displayIndex = 1
    if self.DisplayInfo then
        offset = (offset or self.Scroll:GetScrollValue())
        local dataIndex = offset
        local data = self.DisplayInfo[dataIndex]
        while data and displayIndex <= self.DisplayLines do
            self.DataLine[displayIndex]:SetData(data)
            dataIndex = dataIndex + 1
            displayIndex = displayIndex + 1
            data = self.DisplayInfo[dataIndex]
        end
        self.Scroll:UpdateScrollRange(self.DisplayLines, #self.DisplayInfo, offset)
    end
    VarDataLine.ClearLines(self.DataLine, displayIndex)
end


function Public.Clear(self)
    VarDataLine.ClearLines(self.DataLine, 1)
end


function Scripts.OnMouseWheel(self, delta)
    self.Scroll:Scroll(delta)
end


function Scripts.OnSizeChanged(self, width, height)
    if VarDataLine.UpdateDisplayLines(self) then
        self:Update()
    end
end


function Private.ScrollOnValueChanged(scrollPosition, scrollFrame)
    scrollFrame:Update(Floor(scrollPosition))
end
