-- Copyright 2014-2016 by Adin Hunter Baber, all rights reserved
local _, DataDump = ...
-- lua api
local Floor = math.floor
local Min = math.min
local ipairs = ipairs
-- blizzard api
-- libs
local LazyLib = ClarityLib:GetLib("Clarity4_1.Meta.Lazy")
-- add on api
local VarDisplay = DataDump.NameSpace("VarDisplay", "Display")
-- local name space
local VarManager = DataDump.NameSpace("VarDisplay", "Manager")
local Private = {}
-- locals
local _displays = {}
local _nextIndex = 1


function VarManager.GetDisplay(index)
    index = index or Private.GetNextIndex()
    local display = _displays[index] or VarDisplay.Create(index)

    display.VarInput:Enable()
    display.noSave = nil

    local state = DataDumpData.VarDisplay[index]
    if state then
        display.VarInput:SetText(state.var)
        display:ClearAllPoints()
        local screenX, screenY = UIParent:GetSize()
        local x = (screenX * state.x) / 100
        local y = (screenY * state.y) / 100
        display:SetPoint("Center", UIParent, "BottomLeft", x, y)
        display:SetSize(state.w or 560, state.h or 350)
        display.RefreshButton:SetRefreshRate(state.frames, state.seconds)
    else
        display:SetSize(560, 350)
        display:Clear()
        VarManager.Save(display, true)
    end

    _displays[index] = display
    return display
end


function VarManager.Save(display, force)
    local var = display.VarInput:GetText()
    if (var and var:len() > 0) or force then
        local screenX, screenY = UIParent:GetSize()
        local x, y = display:GetCenter()
        x = Floor(x * 100 / screenX)
        y = Floor(y * 100 / screenY)
        local frames, seconds = display.RefreshButton:GetRefreshRate()
        local w, h = display:GetSize()
        w = Floor(w)
        h = Floor(h)
        local state =
        {
            x = x,
            y = y,
            w = w,
            h = h,
            isShown = display:IsShown(),
            var = not display.noSave and var or "",
            frames = frames > 1 and frames or nil,
            seconds = seconds ~= 1 and seconds or nil,
        }

        DataDumpData.VarDisplay[display.Index] = state
    else
        VarManager.Delete(display)
    end
end


function VarManager.Delete(display)
    DataDumpData.VarDisplay[display.Index] = false
    _nextIndex = Min(_nextIndex, display.Index)
end


function VarManager.ShowAll(forceShow)
    for index,state in ipairs(DataDumpData.VarDisplay) do
        if state then
            local display = VarManager.GetDisplay(index)
            if state.isShown or forceShow then
                display:Show()
            end
        end
    end
end


function VarManager.HideAll()
    for index,state in ipairs(DataDumpData.VarDisplay) do
        if state and state.isShown then
            VarManager.GetDisplay(index):Hide()
        end
    end
end


function Private.GetNextIndex()
    while DataDumpData.VarDisplay[_nextIndex] do
        _nextIndex = _nextIndex + 1
    end
    return _nextIndex
end


function Private.Initialize()
    DataDumpData = DataDumpData or {}
    local oldDisplays = DataDumpData.VarDisplay or {}
    DataDumpData.VarDisplay = {}

    -- compact the saved states
    _nextIndex = 1
    for index,state in ipairs(oldDisplays) do
        if state then
            if state.var and state.var:len() > 0 then
                DataDumpData.VarDisplay[_nextIndex] = state
                _nextIndex = _nextIndex + 1
            end
        end
    end
end


LazyLib.MakeLazy(VarManager, Private.Initialize)
